@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2007-2020 by Ilya Kotov                                 *
+ *   Copyright (C) 2007-2024 by Ilya Kotov                                 *
  *   forkotov02@ya.ru                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -27,6 +27,7 @@ class QObject;
 class QString;
 class QIODevice;
 class QWidget;
+class QDialog;
 class Volume;
 class Decoder;
 class Output;
@@ -65,10 +66,11 @@ public:
      */
     virtual Volume *createVolume() = 0;
     /*!
-     * Shows settings dialog.
+     * Creates settings dialog.
      * @param parent Parent widget.
+     * @return Settings dialog pointer.
      */
-    virtual void showSettings(QWidget *parent) = 0;
+    virtual QDialog *createSettings(QWidget *parent) = 0;
     /*!
      * Shows about dialog.
      * @param parent Parent widget.
