@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2022 by Ilya Kotov                                 *
+ *   Copyright (C) 2008-2024 by Ilya Kotov                                 *
  *   forkotov02@ya.ru                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,11 +21,12 @@
 #define QMMP_H
 
 #include <QString>
+#include <QLoggingCategory>
 #include "qmmp_export.h"
 
 #define QMMP_VERSION_MAJOR 2
-#define QMMP_VERSION_MINOR 1
-#define QMMP_VERSION_PATCH 9
+#define QMMP_VERSION_MINOR 2
+#define QMMP_VERSION_PATCH 1
 #define QMMP_VERSION_STABLE 1
 
 #define QMMP_VERSION_INT (QMMP_VERSION_MAJOR<<16 | QMMP_VERSION_MINOR<<8 | QMMP_VERSION_PATCH)
@@ -39,6 +40,46 @@
 #define QStringToFileName(s) s.toLocal8Bit().constData()
 #endif
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 4, 0)
+
+#include <QLatin1String>
+
+namespace Qt {
+inline namespace Literals {
+inline namespace StringLiterals {
+
+inline QString operator""_s(const char16_t *str, size_t size) noexcept
+{
+    return QString(QStringPrivate(nullptr, const_cast<char16_t *>(str), qsizetype(size)));
+}
+
+constexpr inline QLatin1String operator""_L1(const char *str, size_t size) noexcept
+{
+    return QLatin1String(str, int(size));
+}
+
+inline QByteArray operator""_ba(const char *str, size_t size) noexcept
+{
+    return QByteArray(QByteArrayData(nullptr, const_cast<char *>(str), qsizetype(size)));
+}
+
+} // StringLiterals
+} // Literals
+} // Qt
+
+using QLatin1StringView = QLatin1String;
+
+#endif
+
+using namespace Qt::Literals::StringLiterals;
+
+QMMP_EXPORT Q_DECLARE_LOGGING_CATEGORY(core)
+QMMP_EXPORT Q_DECLARE_LOGGING_CATEGORY(plugin)
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 5, 0)
+#define qCFatal qCCritical
+#endif
+
 
 /*! @brief The Qmmp class stores global settings and enums.
  * @author Ilya Kotov <forkotov02@ya.ru>
