@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2021 by Ilya Kotov                                 *
+ *   Copyright (C) 2008-2024 by Ilya Kotov                                 *
  *   forkotov02@ya.ru                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -56,9 +56,10 @@ public:
      * Subclass should reimplement this function.
      * @param id Command identifier to execute
      * @param args Command arguments
+     * @param currentWorkingDir Current working directory
      * @return Command output
      */
-    virtual QString executeCommand(int id, const QStringList &args) = 0;
+    virtual QString executeCommand(int id, const QStringList &args, const QString &currentWorkingDir) = 0;
     /*!
      * Returns a list of help strings.
      */
