@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2019 by Ilya Kotov                                 *
+ *   Copyright (C) 2008-2024 by Ilya Kotov                                 *
  *   forkotov02@ya.ru                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -65,7 +65,12 @@ public slots:
     /*!
      * Starts playback.
      */
-    void play(qint64 offset = -1);
+    void play();
+    /*!
+     * Starts playback from specified position.
+     * @param offset Start position in ms.
+     */
+    void playFromPosition(qint64 offset);
     /*!
      * Stops playback.
      */
@@ -84,13 +89,14 @@ private slots:
     void updateNextUrl();
     void processState(Qmmp::State state);
     void updateMetaData();
+    void onCurrentTrackRemoved();
 
 private:
     QmmpUiSettings *m_settings;
     PlayListManager *m_pl_manager;
     SoundCore *m_core;
     static MediaPlayer* m_instance;
-    int m_skips;
+    int m_skips = 0;
     QString m_nextUrl;
     QTimer *m_finishTimer;
 };
