@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2013-2014 by Ilya Kotov                                 *
+ *   Copyright (C) 2013-2024 by Ilya Kotov                                 *
  *   forkotov02@ya.ru                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,32 +21,39 @@
 #ifndef PLAYLISTGROUP_H
 #define PLAYLISTGROUP_H
 
+#include <QImage>
 #include "playlisttrack.h"
 #include "playlistitem.h"
+#include "qmmpui_export.h"
 
 class GroupedContainer;
+class QmmpUiSettings;
+class MetaDataHelper;
 
 /** @brief The PlayListTrack class provides a group for use with the PlayListModel class.
  * @author Ilya Kotov <forkotov02@ya.ru>
  */
-class PlayListGroup : public PlayListItem
+class QMMPUI_EXPORT PlayListGroup : public PlayListItem
 {
 public:
     /*!
      * Constructor.
      * @param formattedTitle Title of the group.
      */
-    PlayListGroup(const QString &formattedTitle);
+    explicit PlayListGroup(const QString &formattedTitle);
     /*!
      * Object destructor.
      */
     virtual ~PlayListGroup();
     /*!
      * Returns formatted title of the  group.
-     * @param column Number of column (unused).
+     * @param line Number of title (0 or 1).
      */
-    const QString formattedTitle(int column = 0);
-    const QStringList formattedTitles();
+    QString formattedTitle(int line = 0) const override;
+    /*!
+     * Returns the list of the all avaibale formatted titles (1 or 2).
+     */
+    QStringList formattedTitles() const override;
     /*!
      * Returns \b true if the group contains track \b track.
      * Otherwise returns \b false.
@@ -68,17 +75,42 @@ public:
     /*!
      *  Returns formatted length of the item.
      */
-    const QString formattedLength() { return QString(); }
+    QString formattedDuration() const override;
     /*!
      * Returns \b true.
      */
-    bool isGroup() const;
+    bool isGroup() const override;
+    /*!
+     * Returns path or URL of the first track of the group.
+     */
+    QString firstTrackPath() const;
+    /*!
+     * Returns \b true if the cover is available. Otherwise returns \b false.
+     */
+    bool isCoverLoaded() const;
+    /*!
+     * Returns available cover image.
+     */
+    QImage cover() const override;
+    /*!
+     * Sets cover image for all group.
+     */
+    void setCover(const QImage &cover);
 
 private:
-    QList<PlayListTrack *> trackList; //A list of tracks
-    friend class GroupedContainer;
+    QString formatTitle2() const;
 
-    QString m_name;
+    QList<PlayListTrack *> m_trackList; //A list of tracks
+    QString m_title;
+    mutable QString m_title2;
+
+    bool m_isCoverLoaded = false;
+    QImage m_cover;
+
+    QmmpUiSettings *m_settings;
+    MetaDataHelper *m_helper;
+
+    friend class GroupedContainer;
 };
 
 #endif // PLAYLISTGROUP_H
