@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2022 by Ilya Kotov                                 *
+ *   Copyright (C) 2008-2024 by Ilya Kotov                                 *
  *   forkotov02@ya.ru                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,6 +21,7 @@
 #define PLAYLISTITEM_H
 
 #include <QMap>
+#include <QImage>
 #include <qmmp/qmmp.h>
 #include "qmmpui_export.h"
 
@@ -52,16 +53,16 @@ public:
      * Returns formatted title of the item.
      * @param column Number of column.
      */
-    virtual QString formattedTitle(int column) = 0;
+    virtual QString formattedTitle(int column) const = 0;
     /*!
      * Returns the list of the formatted titles for all columns.
-     * Group separators contain only one title.
+     * Group separators contain only one or two titles.
      */
-    virtual QStringList formattedTitles() = 0;
+    virtual QStringList formattedTitles() const = 0;
     /*!
      *  Returns formatted length of the item.
      */
-    virtual QString formattedLength() = 0;
+    virtual QString formattedDuration() const = 0;
     /*!
      * Returns \b true if the \b PlayListItem is group separator. Otherwise returns \b false.
      */
@@ -71,6 +72,10 @@ public:
      * Default implementation returns -1.
      */
     virtual int trackIndex() const;
+    /*!
+     * Returns available cover image.
+     */
+    virtual QImage cover() const;
 
 private:
     bool m_selected = false;
