@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2006-2016 by Ilya Kotov                                 *
- *   forkotov02@hotmail.ru                                                 *
+ *   Copyright (C) 2006-2024 by Ilya Kotov                                 *
+ *   forkotov02@ya.ru                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -27,9 +27,13 @@
 #include <QQueue>
 #include <QPointer>
 #include <QVector>
+#include <QUrl>
+#include <QSet>
 #include "playlistitem.h"
 #include "playlisttrack.h"
 #include "playlistgroup.h"
+#include "coverloader_p.h"
+#include "qmmpui_export.h"
 
 class FileLoader;
 class PlayState;
@@ -45,60 +49,42 @@ class PlayListTask;
  */
 struct SimpleSelection
 {
-    /*!
-     * Constructs an empty selection model.
-     */
-    SimpleSelection()
-    {
-        m_bottom = -1;
-        m_top = 1;
-    }
-    /*!
-     * Returns \p true if this selection is valid; otherwise returns returns \p false.
-     */
-    inline bool isValid()const
-    {
-        return (m_bottom != -1) && (m_top != -1);
-    }
-    /*!
-     * Prints object state.
-     */
-    inline void dump()const
-    {
-        qWarning("top: %d\tbotom: %d", m_top, m_bottom);
-    }
     /*!
      * Returns number of selected items.
      */
     inline int count()const
     {
-        return m_bottom - m_top + 1;
+        return bottom - top + 1;
     }
-    int m_bottom;               /*!< Bottom of the selection */
-    int m_top;                  /*!< Top of the selection   */
-    QList<int> m_selected_indexes;  /*!< Selected items numbers */
+    int top = 1;                  /*!< Top of the selection   */
+    int bottom = -1;              /*!< Bottom of the selection */
 };
 /*! @brief The PlayListModel class provides a data model for the playlist.
  *
  * @author Vladimir Kuznetsov <vovanec@gmail.com>
- * @author Ilya Kotov <forkotov02@hotmail.ru>
+ * @author Ilya Kotov <forkotov02@ya.ru>
  *
- *         Playlist Structure
- *      -----------------------
- *     | item  | track | item  |
- *     | index | index | type  |
- *      -----------------------
- *     |  0    |       | group |
- *     |  1    |  0    | track |
- *     |  2    |  1    | track |
- *     |  3    |  2    | track |
- *     |  4    |  3    | track |
- *     |  5    |       | group |
- *     |  6    |  4    | track |
- *     |  7    |  5    | track |
- *     |  8    |  6    | track |
+ *              Playlist Structure
+ *      ----------------------------------
+ *     | line  | sub-line | item  | item  |
+ *     | index | index    | index | type  |
+ *      ----------------------------------
+ *     |  0    |  0       |  0    | group |
+ *     |  1    |  1       |  0    | group |
+ *     |  2    |  2       |  0    | group |
+ *      ----------------------------------
+ *     |  3    |  0       |  0    | track |
+ *     |  4    |  0       |  1    | track |
+ *     |  5    |  0       |  2    | track |
+ *      ----------------------------------
+ *     |  6    |  0       |  1    | group |
+ *     |  7    |  1       |  1    | group |
+ *     |  8    |  2       |  1    | group |
+ *      ----------------------------------
+ *     |  9    |  0       |  3    | track |
+ *     | 10    |  0       |  4    | track |
  */
-class PlayListModel : public QObject
+class QMMPUI_EXPORT PlayListModel : public QObject
 {
     Q_OBJECT
 public:
@@ -107,7 +93,7 @@ public:
      * @param name Playlist name.
      * @param parent QObject parent
      */
-    PlayListModel(const QString &name, QObject *parent = 0);
+    explicit PlayListModel(const QString &name, QObject *parent = nullptr);
     /*!
      * Object destructor.
      */
@@ -121,13 +107,17 @@ public:
      */
     void setName(const QString &name);
     /*!
-     * Returns number of items.
+     * Returns number of groups.
      */
-    int count() const;
+    int groupCount() const;
     /*!
      * Returns number of tracks.
      */
     int trackCount() const;
+    /*!
+     * Returns \b true if the model contains no tracks; otherwise returns \b false.
+     */
+    bool isEmpty() const;
     /*!
      * Returns number of columns.
      */
@@ -135,81 +125,64 @@ public:
     /*!
      * Returns the current track.
      */
-    PlayListTrack* currentTrack() const;
+    PlayListTrack *currentTrack() const;
     /*!
-     * Returns the next playing track or 0 if next track is unknown.
+     * Returns the next playing track or \b nullptr if next track is unknown.
      */
-    PlayListTrack* nextTrack() const;
+    PlayListTrack *nextTrack() const;
     /*!
-     * Returns the row of the \b item
+     * Returns index of track or group.
+     * @param item track or group pointer.
      */
-    int indexOf(PlayListItem* item) const;
+    int indexOf(PlayListItem *item) const;
     /*!
-     * Returns the item with the index \b index or 0 if item doesn't exist.
+     * Returns the track with the index \b index or \b nullptr if track doesn't exist.
      */
-    PlayListItem* item(int index) const;
+    PlayListTrack *track(int index) const;
     /*!
-     * Returns the track with the index \b index or 0 if track doesn't exist.
+     * Returns the group with the index \b index or \b nullptr if group doesn't exist.
      */
-    PlayListTrack* track(int index) const;
+    PlayListGroup *group(int index) const;
     /*!
-     * Returns the group with the index \b index or 0 if group doesn't exist.
-     */
-    PlayListGroup* group(int index) const;
-    /*!
-     * Returns index of the current item.
+     * Returns index of the current track or -1 if model is empty.
      */
     int currentIndex() const;
     /*!
-     * Sets current index.
-     * Returns \b false if item with this index doesn't exist, otherwise returns \b true
-     * @param index Number of item.
+     * Sets current track index.
+     * Returns \b false if the track with this index doesn't exist, otherwise returns \b true
+     * @param index Number of the track.
      */
-    bool setCurrent (int index);
+    bool setCurrent(int index);
     /*!
-     * Sets current item to \b item.
+     * Sets current track to \b track.
      * Returns \b true if success, otherwise returns \b false
      */
-    bool setCurrent(PlayListTrack *item);
-    /*!
-     * Returns \b true if a playlist contains track with the index \b index.
-     * Otherwise returns \b false.
-     */
-    bool isTrack(int index) const;
-    /*!
-     * Returns \b true if a playlist contains group separator with the index \b index.
-     * Otherwise returns \b false.
-     */
-    bool isGroup(int index) const;
-    /*!
-     * Returns \b true if item with \b index is selected, otherwise returns \b false
-     */
-    bool isSelected(int index) const;
+    bool setCurrent(PlayListTrack *track);
     /*!
      * Sets the selected state of the item to \b select
-     * @param index Number of item.
+     * @param item Track or group pointer.
      * @param selected Selection state (\b true - select, \b false - unselect)
      */
-    void setSelected(int index, bool selected = true);
+    void setSelected(PlayListItem *item, bool selected = true);
     /*!
      * Sets the selected state of the list of tracks to \b select
      * @param tracks List of tracks.
      * @param selected Selection state (\b true - select, \b false - unselect)
      */
-    void setSelected(QList<PlayListTrack *> tracks, bool selected = true);
+    void setSelected(const QList<PlayListTrack *> &tracks, bool selected = true);
     /*!
      * Sets the selected state of the list of items to \b select
      * @param items List of items.
      * @param selected Selection state (\b true - select, \b false - unselect)
      */
-    void setSelected(QList<PlayListItem *> items, bool selected = true);
+    void setSelected(const QList<PlayListItem *> &items, bool selected = true);
     /*!
      * Sets the selected state of the list of items range.
-     * @param first Firts item in the range.
-     * @param last Last item in the range.
+     * @param firstLine Firts line in the range.
+     * @param lastLine Last line in the range.
      * @param selected Selection state (\b true - select, \b false - unselect).
      */
-    void setSelected(int first, int last, bool selected = true);
+    void setSelectedLines(int firstLine, int lastLine, bool selected = true);
     /*!
      * Advances to the next item. Returns \b false if next iten doesn't exist,
      * otherwise returns \b true
@@ -221,68 +194,116 @@ public:
      */
     bool previous();
     /*!
-     * Returns a list of the items, starting at position \b pos
-     * \param pos First item position.
-     * \param count A number of items. If \b count is -1 (the default), all items from pos are returned.
+     * Returns total line count.
      */
-    QList<PlayListItem *> mid(int pos, int count = -1) const;
+    int lineCount() const;
     /*!
-     *  Moves the item at index position \b from to index position \b to.
+     * Returns item at line \b lineIndex or \b nullptr if item at line \b lineIndex is not available.
      */
-    void moveItems(int from, int to);
+    PlayListItem *itemAtLine(int lineIndex) const;
     /*!
-     * Returns \b true if \b f file is in play queue, otherwise returns \b false.
+     * Returns track at line \b lineIndex or \b nullptr if track at line \b lineIndex is not available.
      */
-    bool isQueued(PlayListTrack* item) const;
+    PlayListTrack *trackAtLine(int lineIndex) const;
     /*!
-     * Returns \b true if play queue is empty,otherwise returns - \b false.
+     * Returns a list of the items, starting at line \b pos
+     * @param pos First item line.
+     * @param count A number of items. If \b count is -1 (the default), all items from pos are returned.
+     */
+    QList<PlayListItem *> itemsAtLines(int pos, int count = -1) const;
+    /*!
+     * Finds line for item \b item. Returns -1 if line for this item is not available.
+     */
+    int findLine(PlayListItem *item) const;
+    /*!
+     * Finds line for track with index \b trackIndex. Returns -1 if the line for this track is not available.
+     */
+    int findLine(int trackIndex) const;
+    /*!
+     * Return sub-index of the line \b lineIndex.
+     */
+    int subIndexOfLine(int lineIndex) const;
+    /*!
+     * Return track index of the line \b lineIndex. Returns -1 if the line does not contain track.
+     */
+    int trackIndexAtLine(int lineIndex) const;
+    /*!
+     * Return \b true if the line \b lineIndex should be painted using alternate color. Otherwise returns \b false.
+     */
+    bool alternateColor(int lineIndex) const;
+    /*!
+     * Returns the number of lines needed to draw the group.
+     */
+    int linesPerGroup() const;
+    /*!
+     *  Moves the track at index position \b from to index position \b to.
+     */
+    void moveTracks(int from, int to);
+    /*!
+     * Returns a list of queued tracks.
+     */
+    const QList<PlayListTrack *> &queuedTracks() const;
+    /*!
+     * Returns \b true if play queue is empty, otherwise returns - \b false.
      */
     bool isEmptyQueue()const;
     /*!
-     * Returns index of \b f file in queue.e
-     */
-    int queuedIndex(PlayListTrack* track) const;
-    /*!
-     * Returns the number of items in the queue
+     * Returns the number of tracks in the queue
      */
     int queueSize() const;
     /*!
-     * Returns \b true if playback stops after \b item, otherwise returns \b false.
+     * Returns \b true if playback stops after \b track, otherwise returns \b false.
      */
-    bool isStopAfter(PlayListItem* item) const;
+    bool isStopAfter(const PlayListItem *track) const;
     /*!
      * Returns current selection(playlist can contain a lot of selections,
-     * this method returns selection which \b row belongs to)
+     * this method returns selection which \b trackIndex belongs to)
      */
-    const SimpleSelection& getSelection(int row);
+    SimpleSelection getSelection(int trackIndex);
     /*!
-     * Returns list with selected items indexes.
+     * Returns a list of the selected lines.
      */
-    QList<int> selectedIndexes() const;
+    QList<int> selectedLines() const;
     /*!
-     * Returns list of \b PlayListItem pointers that are selected.
+     * Selects or unselects group or track at line \b line.
+     * @param line Line number.
+     * @param selected Selection state.
+     */
+    void setSelectedLine(int line, bool selected = true);
+    /*!
+     * Returns list with selected track indexes.
+     */
+    QList<int> selectedTrackIndexes() const;
+    /*!
+     * Returns list of \b PlayListTrack pointers that are selected.
      */
     QList<PlayListTrack *> selectedTracks() const;
     /*!
-     * Returns list of all \b PlayListItem pointers.
+     * Returns list of all \b PlayListTrack pointers.
      */
-    QList<PlayListItem*> items() const;
+    QList<PlayListTrack *> tracks() const;
     /*!
-     * Returns number of first item that selected upper the \b row item.
+     * Returns number of first track index that selected upper the \b trackIndex track.
      */
-    int firstSelectedUpper(int row);
+    int firstSelectedUpper(int trackIndex);
     /*!
-     * Returns number of first item that selected lower the \b row item.
+     * Returns number of first track that selected lower the \b trackIndex track.
      */
-    int firstSelectedLower(int row);
+    int firstSelectedLower(int trackIndex);
     /*!
-     * Returns total lenght in seconds of all songs.
+     * Returns total duration in milliseconds of all songs.
      */
-    int totalLength() const;
+    qint64 totalDuration() const;
     /*!
      * Loads playlist with \b f_name name.
      */
     void loadPlaylist(const QString& f_name);
+    /*!
+     * Loads playlist from content.
+     * @param fmt Playlist format (short name).
+     * @param data Content of the playlist file.
+     */
+    void loadPlaylist(const QString &fmt, const QByteArray &data);
     /*!
      * Saves current songs to the playlist with \b f_name name.
      */
@@ -314,15 +335,15 @@ public:
         GROUP                   /*!< by group name */
     };
     /*!
-     * Converts item index \b index to track index
-     * Returns \b -1 if item with index \b index is not a track or does not exist.
+     * Finds track with index \b trackIndex.
+     * Returns null pointer if playlist does not contain track with index \b track_index.
      */
-    int indexOfTrack(int index) const;
+    PlayListTrack *findTrack(int trackIndex) const;
     /*!
-     * Finds track with index \b track_index.
-     * Return null pointer if playlist does not contain track with index \b track_index.
+     * Finds tracks by string \b str. The search is case insensitive.
+     * Returns a list of \b PlayListItem pointers.
      */
-    PlayListTrack *findTrack(int track_index) const;
+    QList<PlayListItem *> findTracks(const QString &str) const;
     /*!
      * Enum of the playlist update flags.
      */
@@ -343,84 +364,85 @@ signals:
      */
     void listChanged(int flags);
     /*!
-     * Emitted when new track has added.
-     * @param track Pointer of the new playlist track.
+     * Emitted when new tracks have added.
+     * @param tracks A list of pointers of the new playlist tracks.
      */
-    void trackAdded(PlayListTrack *track);
+    void tracksAdded(const QList<PlayListTrack *> &tracks);
     /*!
      * Emitted when playlist name has chanded.
      * @param name New playlist name.
      */
-    void nameChanged(const QString& name);
+    void nameChanged(const QString &name);
     /*!
      * Emitted when playlist loader thread has finished.
      */
     void loaderFinished();
     /*!
-     * Emitted to ensure that the current track is visible.
+     * Tells playlist widget to show track at index \b trackIndex.
      */
-    void currentVisibleRequest();
+    void scrollToRequest(int trackIndex);
     /*!
      * Emitted when sorting by column is finished.
      * @param column Column index.
      * @param reverted Sort direction.
      */
     void sortingByColumnFinished(int column, bool reverted);
+    /*!
+     * Emitted when the current track is removed.
+     */
+    void currentTrackRemoved();
 
 public slots:
     /*!
      * Adds \b track to the playlist.
      */
-    void add(PlayListTrack *track);
+    void addTrack(PlayListTrack *track);
     /*!
      * Adds a list of tracks to the playlist.
      * @param tracks List of tracks.
      */
-    void add(QList <PlayListTrack *> tracks);
+    void addTracks(const QList<PlayListTrack *> &tracks);
     /*!
      * Adds a list of files and directories to the playlist
      * @param path Full path of file or directory.
      */
-    void add(const QString &path);
+    void addPath(const QString &path);
     /*!
      * Adds a list of files and directories to the playlist
      * @param paths Full paths of files and directories.
      */
-    void add(const QStringList &paths);
+    void addPaths(const QStringList &paths);
     /*!
      * Inserts \b track at index position \b index in the playlist.
      */
-    void insert(int index, PlayListTrack *track);
-    /*!
-     * Inserts \b track before playlist item \b before.
-     */
-    void insert(PlayListItem *before, PlayListTrack *track);
+    void insertTrack(int index, PlayListTrack *track);
     /*!
      * Inserts \b tracks at index position \b index in the playlist.
      */
-    void insert(int index, QList <PlayListTrack *> tracks);
+    void insertTracks(int index, const QList<PlayListTrack *> &tracks);
     /*!
-     * Inserts \b tracks after item \b before in the playlist.
+     * Inserts serialized content \b json at index position \b index in the playlist.
+     * May be useful for fast drag-and-drop.
      */
-    void insert(PlayListItem *before, QList <PlayListTrack *> tracks);
+    void insertJson(int index, const QByteArray &json);
     /*!
      * Inserts file or directory at index position \b index in the playlist.
      * @param path Full path of file or directory.
      * @param index Position in the playlist.
      */
-    void insert(int index, const QString &path);
+    void insertPath(int index, const QString &path);
     /*!
      * Adds a list of files and directories at index position \b index in the playlist.
      * @param paths Full paths of files and directories.
      * @param index Position in the playlist.
      */
-    void insert(int index, const QStringList &paths);
+    void insertPaths(int index, const QStringList &paths);
     /*!
      * Adds a list of URLs at index position \b index in the playlist.
      * @param urls A list of URLs.
      * @param index Position in the playlist.
      */
-    void insert(int index, const QList<QUrl> &urls);
+    void insertUrls(int index, const QList<QUrl> &urls);
     /*!
      * Removes all items.
      */
@@ -438,13 +460,21 @@ public slots:
      */
     void removeUnselected();
     /*!
-     * Removes items with \b i index.
+     * Removes track with \b i index.
      */
-    void removeTrack (int i);
+    void removeTrack(int i);
     /*!
-     * Removes item \b item from playlist
+     * Removes track \b track from playlist.
      */
-    void removeTrack (PlayListItem *track);
+    void removeTrack(PlayListTrack *track);
+    /*!
+     * Removes tracks \b items from playlist.
+     */
+    void removeTracks(const QList<PlayListItem *> &items);
+    /*!
+     * Removes tracks \b tracks from playlist.
+     */
+    void removeTracks(const QList<PlayListTrack *> &tracks);
     /*!
      * Inverts selection (selects unselected items and unselects selected items)
      */
@@ -457,32 +487,36 @@ public slots:
      * Shows details for the first selected item.
      * @param parent parent Widget.
      */
-    void showDetails(QWidget *parent = 0);
+    void showDetails(QWidget *parent = nullptr);
     /*!
      * Shows the details for the current song (if any).
      * @param parent parent widget.
      */
-    void showDetailsForCurrent(QWidget *parent = 0);
+    void showDetailsForCurrent(QWidget *parent = nullptr);
     /*!
      * Ensures that the current track is visible.
      */
     void doCurrentVisibleRequest();
+    /*!
+     * Ensures that the playlist track at \b trackIndex is visible.
+     */
+    void scrollTo(int trackIndex);
     /*!
      * Randomly changes items order.
      */
     void randomizeList();
     /*!
-     * Reverces items order.
+     * Reverces tracks order.
      */
     void reverseList();
     /*!
      * Sorts selected items in \b mode sort mode.
      */
-    void sortSelection(int mode);
+    void sortSelection(PlayListModel::SortMode mode);
     /*!
      * Sorts items in \b mode sort mode.
      */
-    void sort(int mode);
+    void sort(PlayListModel::SortMode mode);
     /*!
      * Sorts tracks by the column with index \b column.
      */
@@ -492,9 +526,9 @@ public slots:
      */
     void addToQueue();
     /*!
-     * Adds/removes item \b f to/from playback queue.
+     * Adds/removes track \b t to/from playback queue.
      */
-    void setQueued(PlayListTrack* item);
+    void setQueued(PlayListTrack *t);
     /*!
      * Removes invalid tracks from playlist
      */
@@ -558,19 +592,27 @@ private slots:
 
     void onTaskFinished();
 
+    void updateMetaData(const QStringList &paths);
+
+    void startCoverLoader();
+    void setCover(const QString &path, const QImage &img);
+    void insertTracksInternal(PlayListTrack *before, const QList<PlayListTrack *> &tracks);
+
 private:
-    PlayListTrack* m_current_track;
-    PlayListTrack* m_stop_track;
-    int m_current;
-    SimpleSelection m_selection;  /*!< This flyweight object represents current selection. */
-    QQueue <PlayListTrack*> m_queued_songs; /*!< Songs in play queue. */
+    PlayListTrack *m_current_track = nullptr;
+    PlayListTrack *m_stop_track = nullptr;
+    int m_current = -1;
     PlayState* m_play_state; /*!< Current playing state (Normal or Shuffle) */
-    int m_total_length;
+    qint64 m_total_duration = 0;
     FileLoader *m_loader;
+    CoverLoader *m_coverLoder;
     QString m_name;
     PlayListContainer *m_container;
     QmmpUiSettings *m_ui_settings;
     PlayListTask *m_task;
+    QSet<QString> m_uniquePaths;
 };
 
+Q_DECLARE_METATYPE(PlayListModel::SortMode)
+
 #endif
